﻿-- debug mode off by default
local debugMode = false;
local showOffline, origGuildies;
local isReallyOnline = {};

function GuildWho_Print(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg, 1, 1, 0);
end

-- debug message function, only visible in debug mode
function GuildWho_Debug(msg)
	if debugMode then
		DEFAULT_CHAT_FRAME:AddMessage("## "..msg, 1, 1, 0);
	end
end

-- function to toggle debug mode
function GuildWho_DebugMode()
	if not debugMode then
		debugMode = true;
		GuildWho_Print("GuildWho: Debug mode ON.");
	else
		debugMode = false;
		GuildWho_Print("GuildWho: Debug mode OFF.");
	end
end

function GuildWho_GetLookup(searchStr)
	local lookup, mainChar;
	for i = 1, GetNumGuildMembers(true) do
		local name, _, _, _, _, _, _, note = GetGuildRosterInfo(i);
		if string.lower(searchStr) == string.lower(name) then
			lookup = name;
			GuildWho_Debug("lookup = "..lookup);
			
			-- search string matches an online character, so no need to look any further
			if isReallyOnline[name] then
				return lookup, name;
			end
			
			-- if this character's officer note is EPGPs or blank it's the main, else it's the name of the main
			if string.find(note, "|") or note == "" then
				mainChar = name;
			else
				mainChar = note;
			end
			GuildWho_Debug("mainChar = "..mainChar);
			
			break;
		end
	end
	return lookup, GuildWho_GetAlt(mainChar);
end

function GuildWho_GetAlt(mainChar)
	for i = 1, GetNumGuildMembers(true) do
		local name, _, _, _, _, _, _, note = GetGuildRosterInfo(i);
		if (note == mainChar or name == mainChar) and isReallyOnline[name] then
			GuildWho_Debug("alt = "..name);
			return name;
		end
	end
end

-- match the CHAT_MSG_SYSTEM against online/offline patterns
function GuildWho_ParseSystemMessage(msg)
	local onlineStr = "^"..ERR_FRIEND_ONLINE_SS:format("([^|]+)", ""):gsub("%.", "%%."):gsub("%[%]", ".+").."$";
	local offlineStr = "^"..ERR_FRIEND_OFFLINE_S:format("(%S+)"):gsub("%.", "%%.").."$";
	local name = msg:match(onlineStr);
	if name then
		return name, true;
	end
	return msg:match(offlineStr);
end

function GuildWhoOnline_SlashHandler(msg)
	if not msg or msg == "" then
		GuildWho_Print("Usage:\n"..
						"'/gwho | /online character' - find who character is online as\n"..
						"'/alt character' - list all of character's alts\n"..
						"'/gwho debug' - toggle debug messages.");
	elseif string.lower(msg) == "debug" then
		GuildWho_DebugMode();
	else
		showOffline = GetGuildRosterShowOffline();
		
		-- show offline members to be able to parse them
		SetGuildRosterShowOffline(true);
		
		local lookup, alt = GuildWho_GetLookup(msg);
		
		-- restore old "show offline members" state
		SetGuildRosterShowOffline(showOffline);
		
		-- no character match found
		if not lookup then
			GuildWho_Print("No character named "..string.upper(string.sub(msg, 1, 1))..string.lower(string.sub(msg, 2)).." found in guild.")
			return;
		end
		
		-- print result
		if alt then
			if lookup ~= alt then
				asAlt = " as ".."|Hplayer:"..alt.."|h["..alt.."]|h";
			else
				lookup = "|Hplayer:"..lookup.."|h["..lookup.."]|h";
				asAlt = "";
			end
			GuildWho_Print(lookup.." is online"..asAlt..".");
		else
			GuildWho_Print(lookup.." is offline.");
		end
	end
end

function GuildWhoAlt_SlashHandler(msg)
	if not msg or msg == "" then
		GuildWho_Print("Usage:\n"..
						"'/gwho | /online character' - find who character is online as\n"..
						"'/alt character' - list all of character's alts\n"..
						"'/gwho debug' - toggle debug messages.");
	else
		local lookup, mainChar;
		local hasAlts = false;
		local alts = "";
		
		showOffline = GetGuildRosterShowOffline();
		
		-- show offline members to be able to parse them
		SetGuildRosterShowOffline(true);
		
		for i = 1, GetNumGuildMembers(true) do
			local name, _, _, _, _, _, _, note = GetGuildRosterInfo(i);
			if string.lower(msg) == string.lower(name) then
				lookup = name;
				
				if string.find(note, "|") or note == "" then
					mainChar = name;
				else
					mainChar = note;
				end
				GuildWho_Debug("mainChar = "..mainChar);
			end
		end
		
		for i = 1, GetNumGuildMembers(true) do
			local name, _, _, _, _, _, _, note = GetGuildRosterInfo(i);
			if (note == mainChar or name == mainChar) and name ~= lookup then
				hasAlts = true;
				if isReallyOnline[name] then
					name = "|Hplayer:"..name.."|h["..name.."]|h (online)";
				end
				alts = alts.."\n"..name;
				GuildWho_Debug("Found alt "..name);
			end
		end
		
		-- restore old "show offline members" state
		SetGuildRosterShowOffline(showOffline);
		
		if not lookup then
			GuildWho_Print("No character named "..string.upper(string.sub(msg, 1, 1))..string.lower(string.sub(msg, 2)).." found in guild.")
		elseif not hasAlts then
			GuildWho_Print(lookup.." has no known alts.");
		else
			GuildWho_Print("Known alts of "..lookup..":"..alts);
		end
	end
end

SlashCmdList["GWHO_ONLINE"] = GuildWhoOnline_SlashHandler
SLASH_GWHO_ONLINE1 = "/gwho";
SLASH_GWHO_ONLINE2 = "/online";

SlashCmdList["GWHO_ALT"] = GuildWhoAlt_SlashHandler
SLASH_GWHO_ALT1 = "/alts";

local frame = CreateFrame("Frame");
frame:SetScript("OnEvent", function(frame, event, firstArg)
	if event == "CHAT_MSG_SYSTEM" then
		local name, online = GuildWho_ParseSystemMessage(firstArg);
		showOffline = GetGuildRosterShowOffline();
		SetGuildRosterShowOffline(true);
		for i = 1, GetNumGuildMembers(true) do
			if GetGuildRosterInfo(i) == name then
				inGuild = true;
				break;
			end
		end
		SetGuildRosterShowOffline(showOffline);
		if name then
			GuildWho_Debug("inGuild ("..name..") = "..tostring(inGuild));
			if inGuild then
				if online then
					isReallyOnline[name] = true;
				else
					isReallyOnline[name] = false;
				end
				GuildWho_Debug("isReallyOnline["..name.."] = "..tostring(isReallyOnline[name]));
			end
		end
	elseif event == ("PLAYER_GUILD_UPDATE") then
		if not (GetNumGuildMembers(true) == origGuildies or GetNumGuildMembers(true) == 0) then
			frame:RegisterEvent("GUILD_ROSTER_UPDATE");
		end
		GuildRoster();
	else
		GuildWho_Debug(GetNumGuildMembers(true).. " guild members.");
		local guildies = GetNumGuildMembers(true);
		
		if guildies > 0 then
			origGuildies = guildies;
			showOffline = GetGuildRosterShowOffline();
			frame:UnregisterEvent("GUILD_ROSTER_UPDATE");
			SetGuildRosterShowOffline(true);
			for i = 1, guildies do
				local name, _, _, _, _, _, _, _, online = GetGuildRosterInfo(i);
				isReallyOnline[name] = online;
			end
			SetGuildRosterShowOffline(showOffline);
		end
	end
end);
frame:RegisterEvent("CHAT_MSG_SYSTEM");
frame:RegisterEvent("GUILD_ROSTER_UPDATE");
frame:RegisterEvent("PLAYER_GUILD_UPDATE");